# -*- coding: utf-8 -*-

import logging

import pylons
from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to
import formencode

from ispwebadmin.lib.base import BaseController, render, db, model, my
import webhelpers.paginate

log = logging.getLogger(__name__)

class ValidateNewAlias(formencode.Schema):
    allow_extra_fields = True
    
    # Source can be empty in case of catchall accounts
    source_user = formencode.validators.Regex(r'^[a-z0-9-_\.]*$', max=20, if_empty='')
    destination = formencode.validators.Email(min=1, max=80, not_empty=True)

class ValidateEditAlias(formencode.Schema):
    allow_extra_fields = True
    
    destination = formencode.validators.Email(not_empty=True)

class ValidateAddRecipient(formencode.Schema):
    allow_extra_fields = True
    
    destination = formencode.validators.Email(not_empty=True)


class AliasesController(BaseController):
    """Manage virtual email aliases"""

    def index(self, domain):
        """Show overview of users in a domain"""
        search = request.params.get('search')
        page = request.params.get('page', 1)
        virtual_domain = db.query(model.VirtualDomain).filter_by(name=domain).first()
        if not virtual_domain: abort(404)

        if search:
            sources= virtual_domain.forwardings_sources(search)
        else:
            sources = virtual_domain.forwardings_sources()

        c.search = search
        c.domain = domain
        c.sources = webhelpers.paginate.Page(sources, page=page, search=search)

        return render('/aliases/index.mako')

    def edit(self, domain, source):
        """Show edit form for a certain alias"""
        forwarding = model.Forwarding()
        forwarding.get_by_source(source)
        if not forwarding: abort(404)
        c.forwarding = forwarding
        c.domain = domain
        return render('/aliases/edit.mako')

    def save(self, domain, source):
        """Alter the email alias according to the changes from the edit form"""
        # Check form input
        try:
            fields = my.validate(ValidateEditAlias)
        except formencode.Invalid, e:
            return my.htmlfill(self.edit(domain, source), e)

        alias = db.query(model.VirtualAlias).filter_by(source=source).first()
        if not alias: abort(404)

        alias.destination = fields['destination']
        db.commit()

        log.info("Alias %s changed", alias)
        redirect_to('aliases', domain=domain)

    def addrecipient(self, domain, source):
        """Add a recipient to an alias"""
        # Check form input
        try:
            fields = my.validate(ValidateAddRecipient)
        except formencode.Invalid, e:
            return my.htmlfill(self.edit(domain, source), e)

        forwarding = model.Forwarding()
        forwarding.get_by_source(source)
        if not forwarding: abort(404)
        forwarding.add_recipient(fields['destination'])
        log.info("Recipient %s added to alias %s", fields['destination'], source)
        redirect_to('aliases-edit', domain=domain, source=source)

    def delrecipient(self, domain, source, destination):
        """Delete a recipient from an alias"""
        forwarding = model.Forwarding()
        forwarding.get_by_source(source)
        if not forwarding: abort(404)
        forwarding.remove_recipient(destination)
        log.info("Recipient %s removed from alias %s", destination, source)
        
        # If there are further destinations in the forwarding then send the
        # user to the aliase edit screen. Otherwise send the user to the
        # aliases overview.
        if len(forwarding.aliases)>0:
            redirect_to('aliases-edit', domain=domain, source=source)
        else:
            redirect_to('aliases', domain=domain)

    def save(self, domain, source):
        """Alter the email alias according to the changes from the edit form"""
        # Check form input
        try:
            fields = my.validate(ValidateEditAlias)
        except formencode.Invalid, e:
            return my.htmlfill(self.edit(domain, source), e)

        alias = db.query(model.VirtualAlias).filter_by(source=source).first()
        if not alias: abort(404)

        alias.destination = fields['destination']
        db.commit()

        log.info("Alias %s changed", alias)
        redirect_to('aliases', domain=domain)

    def delete(self, domain, source):
        """Delete a forwarding (containing multiple VirtualAlias rows)"""
        forwarding = model.Forwarding()
        forwarding.get_by_source(source)
        if not forwarding: abort(404)
        forwarding.delete()

        log.info("Alias %s deleted", source)

        # Remove all other aliases pointing to this alias
        aliases = db.query(model.VirtualAlias).filter_by(destination=source)
        for alias in aliases:
            log.info("Removing alias %s which points to the user to be deleted", alias.source)
            db.delete(alias)
        
        redirect_to('aliases', domain=domain)

    def new(self, domain):
        """Show form for creating a new alias"""
        c.domain = domain
        return render('/aliases/new.mako')

    def savenew(self, domain):
        """Create new user alias"""
        # Check form input
        try:
            fields = my.validate(ValidateNewAlias)
        except formencode.Invalid, e:
            return my.htmlfill(self.new(domain), e)

        source ="%s@%s" % (fields['source_user'], domain)

        # Make sure there is no such alias before
        forwarding = model.Forwarding()
        forwarding.get_by_source(source)
        if forwarding:
            log.error("There is already an alias with that email address")
            c.error = u'There is already an alias with that email address.'
            return my.htmlfill(self.new(domain))

        forwarding.source = source
        forwarding.add_recipient(fields['destination'])

        log.info("New alias created: %s", forwarding)

        redirect_to('aliases-edit', domain=domain, source=source)

    def pointing_aliases(self, domain, email):
        """Show a list of aliases that point to this alias address"""
        c.domain = domain
        c.email = email
        c.aliases = db.query(model.VirtualAlias).filter_by(destination=email)
        return render('/users/pointing_aliases.mako')
