import logging

from pylons import request, response, session, config, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from ispwebadmin.lib.base import BaseController, render, db, model
from ispwebadmin.lib import my

import webhelpers.paginate
import formencode

log = logging.getLogger(__name__)

class ValidateNewDomain(formencode.Schema):
    allow_extra_fields = True
    
    name = formencode.validators.Regex(r'^[\w\.-]+\.\w+$', max=50, not_empty=True)


class DomainsController(BaseController):
    """Manage email domains"""
    def index(self):
        """Show list of domains"""
        search = request.params.get('search')
        page = request.params.get('page', 1)
        domains = db.query(model.VirtualDomain)

        if search:
            domains = domains.filter(model.VirtualDomain.name.contains(search))

        c.search = search
        c.domains = webhelpers.paginate.Page(domains, page=page, search=search)
        return render('/domains/index.mako')

    def actions(self, domain):
        """Show the user a menu of what can be done with this domain."""
        domain = db.query(model.VirtualDomain).filter_by(name=domain).first()
        if not domain: abort(404)
        c.domain = domain

        return render('/domains/actions.mako')

    def new(self):
        """Show the form for creating new domains"""
        return render('/domains/new.mako')

    def savenew(self):
        """Create new domain"""
        # Check form input
        try:
            fields = my.validate(ValidateNewDomain)
        except formencode.Invalid, e:
            return my.htmlfill(self.new(), e)

        # Make sure there is no such domain before
        if db.query(model.VirtualDomain).filter_by(name=fields['name']).count() != 0:
            log.error("The domain '%s' exists already.", fields['name'])
            c.error = "The domain '%s' exists already." % fields['name']
            return my.htmlfill(self.new())

        # New domain object
        domain = model.VirtualDomain()
        domain.name = fields['name']
        db.add(domain)
        
        # If 'ispwebadmin.postmaster' is set in the INI file then add forwardings
        # for the 'postmaster' and 'abuse' account automatically.
        postmaster_address = config.get('ispwebadmin.postmaster')
        if postmaster_address:
            log.info('Adding forwardings for abuse and postmaster to %s', postmaster_address)

            for address in ['abuse', 'postmaster']:
                forwarding = model.Forwarding()
                forwarding.source = '%s@%s' % (address, domain.name)
                forwarding.add_recipient(postmaster_address)
        
        db.commit()

        log.info("New domain created: %s", fields['name'])

        redirect_to('domain-actions', domain=fields['name'])

    def delete(self, domain):
        """Delete a domain"""
        domain = db.query(model.VirtualDomain).filter_by(name=domain).first()
        if not domain: abort(404)
        db.delete(domain)
        db.commit()
        log.info('Domain %s deleted' % domain)
        redirect_to('domains')
