# -*- coding: utf-8 -*-

import logging

import pylons
from pylons import request, response, session, tmpl_context as c
from pylons.controllers.util import abort, redirect_to

from ispwebadmin.lib.base import BaseController, render, db, model, my
import webhelpers.paginate
#import webhelpers.html.tags
import formencode
import hashlib
import os
import shutil

log = logging.getLogger(__name__)

class ValidateEditUser(formencode.Schema):
    allow_extra_fields = True
    
    userpart = formencode.validators.Regex(r'^[a-z0-9-_\.]*$', max=20, not_empty=True)
    newpassword = formencode.validators.UnicodeString(min=6, max=30, not_empty=False)

class ValidateNewUser(formencode.Schema):
    allow_extra_fields = True
    
    userpart = formencode.validators.Regex(r'^[a-z0-9-_\.]*$', max=49, not_empty=True)
    password = formencode.validators.UnicodeString(min=6, max=30, not_empty=True)

class UsersController(BaseController):
    """Manage virtual email users"""

    def index(self, domain):
        """Show overview of users in a domain"""
        search = request.params.get('search')
        page = request.params.get('page', 1)
        virtual_domain = db.query(model.VirtualDomain).filter_by(name=domain).first()
        if not virtual_domain: abort(404)

        users = db.query(model.VirtualUser).filter_by(domain=virtual_domain)
        if search:
            users = users.filter(model.VirtualUser.email.contains(search))

        c.search = search
        c.domain = domain
        c.users = webhelpers.paginate.Page(users, page=page, search=search)
        return render('/users/index.mako')

    def new(self, domain):
        """Show form for creating a new user account"""
        c.domain = domain
        c.new_password = my.random_password(9)
        return render('/users/new.mako')

    def edit(self, domain, email):
        """Show edit form for a certain user"""
        user = db.query(model.VirtualUser).filter_by(email=email).first()
        if not user: abort(404)
        
        c.domain = domain
        c.user = user
        return render('/users/edit.mako')

    def delete(self, domain, email):
        """Delete user account"""
        user = db.query(model.VirtualUser).filter_by(email=email).first()
        if not user: abort(404)
        log.info("Deleting user: %s", user)
        
        c.email = user.email
        c.maildir = user.get_maildir_path()
        c.domain = domain

        db.delete(user)
        
        # Remove any aliases pointing to this email account
        c.deleted_aliases = []
        aliases = db.query(model.VirtualAlias).filter_by(destination=email)
        for alias in aliases:
            log.info("Removing alias %s which points to the user to be deleted", alias.source)
            c.deleted_aliases.append(alias.source)
            db.delete(alias)
        
        db.commit()

        return render('/users/deleted.mako')

    def save(self, domain, email):
        """Alter the email account according to the changes from the edit form"""
        # Check form input
        try:
            fields = my.validate(ValidateEditUser)
        except formencode.Invalid, e:
            return my.htmlfill(self.edit(domain, email), e)

        # Load user object
        user = db.query(model.VirtualUser).filter_by(email=email).first()
        if not user: abort(404)

        # Password changed?
        if fields.get('newpassword'):
            hashed_password = hashlib.md5(fields['newpassword']).hexdigest()
            log.info('User %s gets new password (MD5 hash: %s)', user, hashed_password)
            user.password = hashed_password

        if user.userpart != fields['userpart']:
            log.info('User %s gets a new userpart: %s', user, fields['userpart'])

            new_email = '%s@%s' % (fields['userpart'], user.domain.name)
            log.debug("User's (new) email address: %s", new_email)

            # Is the new login already taken?
            if db.query(model.VirtualUser).filter_by(email=new_email).count()>0:
                abort(403, "The email address '%s' exists already." % new_email)

            # Preparing rename of the user's maildir
            old_directory = user.get_maildir_path()
            new_directory = user.get_maildir_path(userpart=fields['userpart'])
            log.debug("Renaming maildir. Old maildir=%s ; New maildir: %s",
                old_directory, new_directory)
            # Is the new directory location available?
            if os.path.exists(new_directory):
                log.info('Rename failed. Target directory "%s" exists already', new_directory)
                abort(403, "The new mail directory '%s' exists already." % new_directory)

            # Rename the maildir to the new location
            try:
                shutil.move(old_directory, new_directory)
            except IOError:
                log.error('Renaming of maildir failed.')

            user.email = new_email

        db.commit()
        log.info("User %s changed", user)
        redirect_to('users', domain=domain)

    def savenew(self, domain):
        """Create new user account"""
        # Check form input
        try:
            fields = my.validate(ValidateNewUser)
        except formencode.Invalid, e:
            return my.htmlfill(self.new(domain), e)

        email="%s@%s" % (fields['userpart'], domain)

        # Make sure there is no such user before
        if db.query(model.VirtualUser).filter_by(email=email).count() != 0:
            log.error("There is already another user with that email address")
            c.error = u'There is already another user with that email address.'
            return my.htmlfill(self.new(domain))

        # New user object
        user = model.VirtualUser()

        user.password=unicode(hashlib.md5(fields['password']).hexdigest())
        user.domain=db.query(model.VirtualDomain).filter_by(name=domain).one()
        user.email=email

        db.add(user)
        db.commit()
        log.info("New user created: %s", user)

        redirect_to('users', domain=domain)

    def pointing_aliases(self, domain, email):
        """Show a list of aliases that point to this email address"""
        c.domain = domain
        c.email = email
        c.aliases = db.query(model.VirtualAlias).filter_by(destination=email)
        return render('/users/pointing_aliases.mako')
