"""The application's model objects"""
import sqlalchemy as sa
import pylons
import os

from sqlalchemy import orm

from ispwebadmin.model import meta

def init_model(engine):
    """Call me before using any of the tables or classes in the model"""
    meta.Session.configure(bind=engine)
    meta.engine = engine


class MyOrm(object):
    pass


# Virtual users
virtual_users_table = sa.Table('virtual_users', meta.metadata,
    sa.Column('id', sa.types.Integer, primary_key=True),
    sa.Column('domain_id', sa.types.Integer, sa.ForeignKey('virtual_domains.id')),
    sa.Column('email', sa.types.Unicode(100), nullable=False),
    sa.Column('password', sa.types.Unicode(32)),
    )

class VirtualUser(MyOrm):
    def __repr__(self):
        result = u"VirtualUser (#%s): Email=%s Domain=%s" % \
            (self.id, self.email, self.domain.name)
        return result.encode('utf-8')
        
    @property
    def userpart(self):
        """Return the part before the '@' of an email address"""
        return self.email.split('@')[0]

    def get_maildir_path(self, userpart=None):
        """Return the path to this user's mail directory on disk"""
        # If userpart parameter is given then return the path for a
        # potential maildir with another userpart.
        # (userpart: the part of the email address before the '@')
        # This is used when maildirs are renamed.
        if userpart: login_name = userpart
        else: login_name = self.userpart
        return os.path.join(
            pylons.config['ispwebadmin.maildir_location'],
            self.domain.name,
            login_name,
            )

# Virtual domains
virtual_domains_table = sa.Table('virtual_domains', meta.metadata,
    sa.Column('id', sa.types.Integer, primary_key=True),
    sa.Column('name', sa.types.Unicode(50), nullable=False),
    )

class VirtualDomain(MyOrm):
    def __repr__(self):
        result = u"VirtualDomain (#%s): %s" % (self.id, self.name)
        return result.encode('utf-8')

    def forwardings_sources(self, search=None):
        """Returns a list of unique source email addresses of all aliases of this domain.
        
        An alias to different destination addresses consists of several
        rows in the database. Example:
        
        a@domain -> b@domain, c@domain, d@domain
        
        Looks like this in the database:
        
        a@domain -> b@domain
        a@domain -> c@domain
        a@dmoain -> d@domain
        
        So this method returns just one a@domain.
        
        If 'search' is specified then filter by this query."""
        select = sa.select([VirtualAlias.source],
            whereclause=(VirtualAlias.domain==self),
            order_by=VirtualAlias.source
            ).distinct()
        if search:
            select = select.where(VirtualAlias.source.contains(search))
        rows = meta.Session.execute(select)
        return [row[0] for row in rows]


# Virtual aliases
virtual_aliases_table = sa.Table('virtual_aliases', meta.metadata,
    sa.Column('id', sa.types.Integer, primary_key=True),
    sa.Column('domain_id', sa.types.Integer, sa.ForeignKey('virtual_domains.id')),
    sa.Column('source', sa.types.Unicode(40), nullable=False),
    sa.Column('destination', sa.types.Unicode(80), nullable=False),
    )

class VirtualAlias(MyOrm):
    def __repr__(self):
        result = "VirtualAlias (#%s): %s -> %s (domain=%s)" % \
            (self.id, self.source, self.destination, self.domain.name)
        return result.encode('utf-8')

    @property
    def userpart(self):
        """Return the part before the '@' of an alias's source address"""
        return self.source.split('@')[0]


# Administrative users
admins_table = sa.Table('admins', meta.metadata,
    sa.Column('id', sa.types.Integer, primary_key=True),
    sa.Column('username', sa.types.Unicode(40), nullable=False),
    sa.Column('pwhash', sa.types.Unicode(32), nullable=False),
    )

class Admin(MyOrm):
    def __repr__(self):
        return "Admin (#%s): %s" % (self.id, self.username)


# ORM mappers
orm.mapper(VirtualUser, virtual_users_table, order_by=virtual_users_table.c.email)
orm.mapper(VirtualAlias, virtual_aliases_table,
    order_by=[virtual_aliases_table.c.source, virtual_aliases_table.c.destination])
orm.mapper(VirtualDomain, virtual_domains_table, order_by=virtual_domains_table.c.name,
    properties={
        'users':orm.relation(
            VirtualUser,
            backref=orm.backref('domain', uselist=False),
            cascade="all, delete, delete-orphan"
            ),
        'aliases':orm.relation(
            VirtualAlias,
            backref=orm.backref('domain', uselist=False),
            cascade="all, delete, delete-orphan"
            ),
        }
    )
orm.mapper(Admin, admins_table)


class Forwarding(object):
    """Abstract object for a certain email forwarding.
    
    One email forwarding may consist of several VirtualAlias entries.
    Thus this object provides an abstract representation of a forwarding.
    
    An alias to different destination addresses consists of several
    rows in the database. Example:
    
    a@domain -> b@domain, c@domain, d@domain
    
    Looks like this in the database:
    
    a@domain -> b@domain
    a@domain -> c@domain
    a@dmoain -> d@domain"""

    # TODO: perhaps support multiple comma-seperated destinations in one entry?

    def __init__(self):
        """Create a new empty Forwarding object"""
        self.source = None
        self.aliases = []

    def __repr__(self):
        """Return representation of a forwarding"""
        return "Forwarding: %s -> %s" % (
            self.source,
            ','.join([virtualalias.destination for virtualalias in self.aliases])
            )

    def __nonzero__(self):
        """Return True if the forwarding contains something useful"""
        if self.source:
            return True
        else:
            return False

    def get_by_source(self, source):
        """Returns VirtualAlias objects with a certain source email address"""
        aliases = meta.Session.query(VirtualAlias).filter_by(source=source)
        if aliases.count()>0:
            self.source = source
            self.aliases = aliases.all()
        else:
            self.source = None
            self.aliases = []

    def add_recipient(self, destination):
        """Add a recipient to this forwarding"""
        domain_name = self.source.split('@')[1]
        domain = meta.Session.query(VirtualDomain).filter_by(name=domain_name).first()
        assert domain

        # Make sure that there is no such recipient yet
        if meta.Session.query(VirtualAlias).filter_by(source=self.source, destination=destination).count()>0:
            # TODO: don't make this go unnoticed
            return

        alias = VirtualAlias()
        alias.source = self.source
        alias.domain = domain
        alias.destination = destination
        meta.Session.add(alias)
        meta.Session.commit()
        
        # Refresh object
        self.get_by_source(self.source)

    def remove_recipient(self, destination):
        """Remove a recipient from this forwarding"""
        alias = meta.Session.query(VirtualAlias).filter_by(source=self.source, destination=destination).first()
        if alias:
            meta.Session.delete(alias)
            meta.Session.commit()

        # Refresh object
        self.get_by_source(self.source)

    def delete(self):
        """Remove a forwarding (containing multiple VirtualAlias rows)"""
        for alias in self.aliases:
            meta.Session.delete(alias)
        meta.Session.commit()
